
/* Kirk Christiansen */
/* NAIVE2.C          */

/* This module contains heuristics */
/* for the second naive-style of   */
/* attack.                         */

#ifndef _NAIVE2
/************************************************************************/
#include <stdio.h>
#include <stdlib.h>    /* random functions */
#include "tankvars.c"
#include "tanksrch.c"
/************************************************************************/
/* Function Prototypes */
void naive2_hunt_for_tank(int tanknum);
void naive2_attack(int attacker);
/************************************************************************/
void naive2_attack(int attacker)
/* This second naive heuristic style, adds the following techniques  */
/* to the tank's movement pruning:  The tank will now scan to its    */
/* left and right, as well as straight ahead.  This will give the    */
/* the tank a better choice of movement and will greatly reduce the  */
/* amount of possible moves it might spend looking for an opponent   */
/* tank.  The tank will also concentrate on a more direct search of  */
/* an opponent tank. The tank will not stop and rotate as much as it */
/* did in the first naive-style because it is now scaning left and   */
/* right as it proceeds along.  Also, the tank has a better heuristic */
/* to negotiating obstacles.  The tank will now move up to wall of an */
/* obstacle, and then try to go around it.  In the first naive style, */
/* if the tank detected an obstacle between itself and its enemy, the */
/* tank tried to negotiate the obstacle right then, by rotating left, */
/* moving forward a couple of times, rotating right and finally moving */
/* forward a couple of more times.  Now its goes up to the obstacle    */
/* first.                                                              */
{
 int tankfoundleft = FALSE;    /* is there a tank to the left?    */
 int tankfoundright = FALSE;   /* is there a tank to the right?   */
 int tankfound = FALSE;        /* is there a tank straight ahead? */

 int obstacle  = FALSE;
 int movepathblocked = FALSE;
 int enemy=-1;
 int distance=0;
 int direction;
 int dieroll=0;

 /* The tank must now check in 3 directions.  */
 /* First, the tank will scan straight ahead. */
 direction = tanks[attacker].pos;
 scan_for_tank (attacker, direction, &tankfound,&enemy, &distance);

 /* If no tank was found, the tank will check to its right hand side */
 /* and if no tank there, it will scan to its left hand side.        */
 if (!tankfound)
  {
    direction = right_of(direction);
    scan_for_tank (attacker, direction, &tankfoundright,&enemy, &distance);

     /* If enemy is to the right, rotate towards it */
     if (tankfoundright)
       {
	add_buffer(attacker,RIGHTARROW);  /* or Turn right */
	add_buffer(attacker,RIGHTARROW);  /* or Turn right */
	return;
       }
     else /* Check the left side of tank for enemy */
       {
	 direction = opposite_of(direction); /* from right to left */
	 scan_for_tank (attacker, direction, &tankfoundleft,
			&enemy, &distance);
	 if (tankfoundleft)
	   {
	    add_buffer(attacker,LEFTARROW);  /* Turn left */
	    add_buffer(attacker,LEFTARROW);  /* Turn left */
	    return;
	   }
	 else    /* no tanks in all 3 directions, so search */
	  {
	   naive2_hunt_for_tank(attacker);
	  }
       }
  }

 /* If a tank is found, calculate some moves */

 else if (tankfound)  /* straight ahead */
  {
    /* Check to see if obstacle is between the two tanks */
    obstacle = check_projpath_for_obstacle (tanks[attacker].xcor,
	       tanks[attacker].ycor,tanks[enemy].xcor,tanks[enemy].ycor,
	       tanks[attacker].pos);

    if (!obstacle)   /* No obstacle in firing or movement path */
     {
      if (tanks[attacker].armor > 4) /* tank's armor condition is good */
	{
	 if (distance != ADJACENT) /* tanks are not next to each other */
	  {
	   add_buffer(attacker,SPACE);  /* fire */
	   add_buffer(attacker,UPARROW);   /* move at tank */
	   add_buffer(attacker,UPARROW);   /* move at tank */
	  }
	 else /* tanks are ADJACENT */
	  {
	   add_buffer(attacker,SPACE);  /* fire */
	   add_buffer(attacker,DOWNARROW); /* back away */
	   add_buffer(attacker,DOWNARROW); /* back away */
	  }
	}

	 /* else if (attack tank has poor armor condition) */
	 else if (tanks[attacker].armor <= 4)
	  {
	   if (distance != ADJACENT)
	     {
	      add_buffer(attacker,SPACE);
	      add_buffer(attacker,DOWNARROW);
	      add_buffer(attacker,SPACE);
	     }
	   else /* tanks are ADJACENT */
	     {
	      add_buffer(attacker,DOWNARROW);
	      add_buffer(attacker,SPACE);
	      add_buffer(attacker,DOWNARROW);
	      add_buffer(attacker,DOWNARROW);
	     }
	  }
     } /* end if (!obstacle) */

     /* else if (an obstacle is in path between tanks) */
     else if (obstacle)
     {
       /* See if obstacle is in tank's move path */
       movepathblocked = check_movepath_for_obstacle(tanks[attacker].xcor,
			       tanks[attacker].ycor, tanks[attacker].pos);

       if (movepathblocked)
       {
	 /* Try to go around the obstacle */
	 dieroll = (rand()%2);                  /* either left or right*/

	 if (dieroll)
	  {
	   add_buffer(attacker,LEFTARROW);  /* Turn left */
	   add_buffer(attacker,LEFTARROW);  /* Turn left */
	  }
	 else
	  {
	   add_buffer(attacker,RIGHTARROW);  /* or Turn right */
	   add_buffer(attacker,RIGHTARROW);  /* or Turn right */
	  }

	 add_buffer(attacker,UPARROW);
	 add_buffer(attacker,UPARROW);    /* Move forwards */
	 add_buffer(attacker,UPARROW);    /* several times */
	 add_buffer(attacker,UPARROW);
	 add_buffer(attacker,UPARROW);
	 add_buffer(attacker,UPARROW);

	 if (dieroll)
	   {
	    add_buffer(attacker,RIGHTARROW);  /* Turn Right   */
	    add_buffer(attacker,RIGHTARROW);  /* Turn Right   */
	   }
	 else                                         /*     or       */
	   {
	    add_buffer(attacker,LEFTARROW);   /* Turn Left    */
	    add_buffer(attacker,LEFTARROW);   /* Turn Left    */
	   }

	 add_buffer(attacker,UPARROW);     /* Move forwards */
	 add_buffer(attacker,UPARROW);
       }
       else /* obstacle is far from tank, so tank can move forwards */
       {
	 add_buffer(attacker,UPARROW);
	 add_buffer(attacker,UPARROW);
       }
     }
  } /* end if (tankfound) */
}
/*************************************************************************/
void naive2_hunt_for_tank(int tanknum)
/* This function simply gets as input a tank, and calculates a path */
/* movement for that tank to follow.  This is done by checking to   */
/* see if a wall or other obstacles are in its path.                */
{
 int dieroll=0;

   /* If (tank cannot move forward due to an obstacle) */
 if (check_movepath_for_obstacle(tanks[tanknum].xcor,tanks[tanknum].ycor,
				 tanks[tanknum].pos))
   {
     /* pick a random turn away from the obstacle */
     dieroll = (rand() % 3);

     /* 2 out of 3 times make a right hand rotate */
     if (dieroll)
      {
       add_buffer(tanknum,RIGHTARROW);  /* Turn Right   */
       add_buffer(tanknum,RIGHTARROW);  /* Turn Right   */
      }
     else
      {
       add_buffer(tanknum,LEFTARROW);  /* Turn Left  */
       add_buffer(tanknum,LEFTARROW);
      }
   }

 else  /* No wall in tank's move path, so continue a search */
   {
    dieroll = ((rand() % 42) + 1);

    /* Movement based on percentages */
    if (dieroll <= 35)  /* 83% chance of moving forward */
       tanks[tanknum].currentinput = UPARROW;

    /* 4% chance to make a turn (2% for each way)  */
    else if ((dieroll == 36) && (tanks[tanknum].lastinput != LEFTARROW))
     {
       add_buffer(tanknum,RIGHTARROW);  /* Turn Right   */
       add_buffer(tanknum,RIGHTARROW);  /* Turn Right   */
     }
    else if ((dieroll == 37) && (tanks[tanknum].lastinput != RIGHTARROW))
     {
       add_buffer(tanknum,LEFTARROW);  /* Turn Left  */
       add_buffer(tanknum,LEFTARROW);
     }

    else   /* 12% chance do nothing this turn */
      tanks[tanknum].currentinput = NOMOVE;
   }
}
/************************************************************************/
#define _NAIVE2
#endif
