
/* Kirk Christiansen */
/* PATROL.C           */

/*  This 'C' module  contains  the functions  */
/* for a PATROL-style of computer tank attack. */
/* These heuristics functions will find a     */
/* move or series of moves for the tank to    */
/* follow.  A series of moves will be stored  */
/* in the tank's input buffer.                */

#ifndef _PATROL.C
/*************************************************************************/
#include <stdio.h>
#include <stdlib.h>    /* random functions */
#include "tankvars.c"
#include "tanksrch.c"
/*************************************************************************/
/* Function Prototypes */
void patrol_hunt_for_tank(int tanknum);
void patrol_attack(int attacker);
/*************************************************************************/
void patrol_attack(int attacker)
/* This function contains the heuristics for the patrol style of       */
/*  tank attacking.  A computer tank with these heuristics will       */
/*  do a brief scan in front of its turret and look for an enemy      */
/*  tank. If it sees one, it will fire; and depending on its current  */
/*  armor condition it will move at the tank, or away from it.        */
/* Part of this heuristic includes making sure an obstacle is not     */
/*  between itself and the enemy tank, if there is one, the tank      */
/*  will attempt to negociate the obstacle by a series of moves.      */
/* If the tank sees no enemy tank in front of its turret, it will     */
/*  resort to a tank hunting search which is based on percentages     */
/*  calculated on different types of moves available.                 */
/* The function is passed in the tanks array, and the array subscript */
/*  number of the tank that is using the heuristics (attacker).       */
{
 int tankfound = FALSE;
 int obstacle  = FALSE;
 int enemy=-1;
 int distance=0;
 int direction;

 /* Set the direction in which the tank will look */
 direction = tanks[attacker].pos;

 /* Call a function to determine if an enemy tank can be seen */
 /* in the direction of the tanks turret, return TRUE if yes  */
 scan_for_tank (attacker, direction, &tankfound,
		&enemy, &distance);


 /**** Begin to determine a move to make ****/

 if (tankfound)  /* Enemy tank is in direction of turret */
  {
    /* Check to see if obstacle is between the two tanks */
    obstacle = check_projpath_for_obstacle (tanks[attacker].xcor,
	       tanks[attacker].ycor,tanks[enemy].xcor,tanks[enemy].ycor,
	       tanks[attacker].pos);

    if (!obstacle)   /* No obstacle in firing or movement path */
     {
      if (tanks[attacker].armor > 4) /* tank's armor condition is good */
	{
	 if (distance != ADJACENT) /* tanks are not next to each other */
	  {
	   add_buffer(attacker,SPACE);  /* fire */
	   add_buffer(attacker,UPARROW);   /* move at tank */
	   add_buffer(attacker,UPARROW);   /* move at tank */
	   add_buffer(attacker,UPARROW);   /* move at tank */
	  }
	 else /* tanks are ADJACENT */
	  {
	   add_buffer(attacker,SPACE);  /* fire */
	   add_buffer(attacker,DOWNARROW); /* back away */
	   add_buffer(attacker,DOWNARROW); /* back away */
	   add_buffer(attacker,DOWNARROW); /* back away */
	  }
	}

	 /* else if (attack tank has poor armor condition) */
	 else if (tanks[attacker].armor <= 4)
	  {
	   if (distance != ADJACENT)
	     {
	      add_buffer(attacker,SPACE);
	      add_buffer(attacker,DOWNARROW);
	      add_buffer(attacker,DOWNARROW);
	     }
	   else /* tanks are ADJACENT */
	     {
	      add_buffer(attacker,SPACE);
	      add_buffer(attacker,DOWNARROW);
	      add_buffer(attacker,DOWNARROW);
	      add_buffer(attacker,DOWNARROW);
	     }
	  }
     } /* end if (!obstacle) */

     /* else if (an obstacle is in path between tanks) */
     else if (obstacle)
     {
       /* Try to go around the obstacle */
       add_buffer(attacker,LEFTARROW);  /* Turn left */
       add_buffer(attacker,LEFTARROW);  /* by two turns */
       add_buffer(attacker,UPARROW);
       add_buffer(attacker,UPARROW);    /* Move forwards */
       add_buffer(attacker,UPARROW);    /* several times */
       add_buffer(attacker,UPARROW);
       add_buffer(attacker,UPARROW);
       add_buffer(attacker,UPARROW);
       add_buffer(attacker,RIGHTARROW);  /* Turn Right    */
       add_buffer(attacker,RIGHTARROW);  /* Turn Right    */
       add_buffer(attacker,UPARROW);     /* Move forwards */
       add_buffer(attacker,UPARROW);
     }
  } /* end if (tankfound) */


  else /* an enemy tank is not found so call a function that */
       /* will calculate a move based on percentages.        */
     {
       patrol_hunt_for_tank(attacker);
     }
}
/*************************************************************************/
void patrol_hunt_for_tank(int tanknum)
/* This function simply gets as input a tank, and calculates a path */
/* movement for that tank to follow.  This is done by checking to   */
/* see if a wall or other obstacles are in its path.                */
{
 int dieroll=0;

   /* If (tank cannot move forward due to an obstacle) */
 if (check_movepath_for_obstacle(tanks[tanknum].xcor,
				 tanks[tanknum].ycor,tanks[tanknum].pos))
   {
     /* pick a random turn away from the obstacle */
     dieroll = (rand() % 3);

     /* 2 out of 3 times make a right hand rotate */
     if (dieroll)
       {
	add_buffer(tanknum,RIGHTARROW);  /* Turn Right    */
	add_buffer(tanknum,RIGHTARROW);
       }
     else
       {
	add_buffer(tanknum,LEFTARROW);  /* Turn Left  */
	add_buffer(tanknum,LEFTARROW);
       }
   }

 else  /* No wall in tank's move path, so continue a search */
   {
    dieroll = ((rand() % 110) + 1);

    /* movement based on percentages */
    if (dieroll <= 17)  /* 17/25 chance of moving forward */
      tanks[tanknum].currentinput = UPARROW;

    else if (dieroll == 18)// && (tanks[tanknum].lastinput != LEFTARROW))
      {
	add_buffer(tanknum,RIGHTARROW);  /* Turn Right    */
	add_buffer(tanknum,RIGHTARROW);
      }
    else if (dieroll == 19)// && (tanks[tanknum].lastinput != LEFTARROW))
      {
	add_buffer(tanknum,LEFTARROW);  /* Turn Right    */
	add_buffer(tanknum,LEFTARROW);
      }
    else
      tanks[tanknum].currentinput = NOMOVE;
   }
}
/*************************************************************************/
#define _PATROL.C
#endif


