
/* Kirk Christiansen           */
/* TANKSRCH.C                  */

/* This unit of 'C' functions are for tank searching */
/* that is done by an attacking tank.  Such as determining */
/* the distance of an opponent tank, if it has projectiles */
/* heading towards the tank, and if the tank is attacking. */

#ifndef  _TANKSRCH
/*************************************************************************/
#include <time.h>
#include <stdlib.h>      /* for random function */
#include "kc_keybd.c"
#include "tankvars.c"
#include "kcgrphcs.c"

#define   CHECKRATE  5  /* 'PROJ PATH CHECKRATE' */
			/* the distance between checks for searching */
			/* a projectile path for obstacles */
/*************************************************************************/
/* This structure is used for a certainty factor storage for each */
/* possible move direction.  Terminator mode uses this structure */
  struct ways_you_can_move
  {
    int up;     /* MUST CHANGE THIS to include all 8 directions */
    int down;
    int left;
    int right;
  };
/*************************************************************************/
/* Function Prototypes */
int is_obstacle (int x, int y);
int getdistance (int x1, int y1, int x2, int y2);
int convertdistance (int dist);
int check_projpath_for_obstacle (int x1, int y1, int x2, int y2, int dir);
int check_movepath_for_obstacle (int x, int y, int dir);
int search_tankbox (int x, int y, int box_x, int box_y);
void scan_for_tank(int looktank,int lookdir,int *tankfound,int *enemynum,
		   int *distance);
void locate_enemy(int *x, int *y,int attacker);
int right_of(int dir);
int left_of(int dir);
int opposite_of(int dir);
void turn_tank (int tanknum, int dir);
void turn_tank_up (int tanknum);
void turn_tank_down (int tanknum);
void turn_tank_left (int tanknum);
void turn_tank_right (int tanknum);
void negotiate_obstacle (int tanknum);
int get_width_obstacle (int x, int y, int pos, int dir);
int examine_terrain(int x, int y);
int closest_obstacle_is(int x, int y);
void randomattack (int tanknum);
/*************************************************************************/
int is_obstacle (int x, int y)
/* Returns TRUE if the pixel at (x,y) is part of an obstacle */
{
  int i,pixcolor;

  pixcolor = getpixel (x,y,screen_buffer);

 /* THIS BELOW LOOP EATS UP MASSIVE CYCLES ! */
 /* cuz its getting called in dist. intervals for each check */

 /* this statement should speed it up */
 if (pixcolor==BLACK)
   return FALSE;

 for (i=0;i<MAX_OBSTACLE_CLRS;i++)
   {
    if (pixcolor==obstacle_color_array[i])
      return TRUE;
   }

 return FALSE;
}
/*************************************************************************/
int getdistance (int x1, int y1, int x2, int y2)
/* This function takes in two (x,y) coordinate pairs and returns */
/* the distance between them in the form of x distance or y distance */
/* not using a combinational slope formula to avoid division */
{
 int dist1=0, dist2=0;     /* initialize to 0 */

 if (x1 >= x2)             /* calculate a positive distance */
    dist1 = (x1-x2);
 else
    dist1 = (x2-x1);

 if (y1 >= y2)
    dist2 = (y1-y2);
 else
    dist2 = (y2-y1);


 if (dist1 >= dist2)   /* return whichever is greater, the x distance */
   return dist1;       /* (dist1), or the y distance (dist2).         */
 else
   return dist2;
}
/*************************************************************************/
int convertdistance (int dist)
/* This function simply converts a distance into distanttype form */
/* (i.e.  ADJACENT,CLOSE,MEDIUM,FAR distance).                    */
{
     /* Put distance in enumeration type terms */
     if (dist <= ADJACENTDISTANCE)
       return ADJACENT;
     else if (dist <= CLOSEDISTANCE)
       return CLOSE;
     else if (dist <= MEDIUMDISTANCE)
       return MEDIUM;
     else
       return FAR;
}
/*************************************************************************/
int check_projpath_for_obstacle (int x1, int y1, int x2, int y2, int dir)
/* This function checks to see if a wall or obstacle is between    */
/* the firing tank (x1,y1) and the target tank (x2,y2). It returns */
/* TRUE if an obstacle exists between the two tanks.               */
{
 int i;
 int obstacle = FALSE;

 /* NOTE ON BELOW LOOPS...I might be able to leave out is_obstacle */
 /* case of searching the middle part of proj path...since im checking */
 /* the bounds.....but if a single pixel can repeatively stop a proj */
 /* then I'll need to keep the function call in. */

 switch (dir)
 {
  case UP     : for (i=(y1-TS-2);((i >= y2) && (!obstacle)); i-=CHECKRATE)
		     {
		       obstacle = ((is_obstacle(x1,i)) ||
				(is_obstacle(x1-1,i)) ||
				(is_obstacle(x1+1,i)));
		     }
		break;
  case RIGHT  : for (i=(x1+TS+2);((i <= x2) && (!obstacle)); i+=CHECKRATE)
		     {
		       obstacle = ((is_obstacle(i,y1)) ||
				(is_obstacle(i,y1-1)) ||
				(is_obstacle(i,y1+1)));
		     }
		break;
  case DOWN   : for (i=(y1+TS+2);((i <= y2) && (!obstacle)); i+=CHECKRATE)
		     {
		       obstacle = ((is_obstacle(x1,i)) ||
				(is_obstacle(x1-1,i)) ||
				(is_obstacle(x1+1,i)));
		     }
		break;
  case LEFT   : for (i=(x1-TS-2);((i >= x2) && (!obstacle)); i-=CHECKRATE)
		     {
		       obstacle = ((is_obstacle(i,y1)) ||
				(is_obstacle(i,y1-1)) ||
				(is_obstacle(i,y1+1)));
		     }
		break;
 }

 return obstacle;
}
/*************************************************************************/
int check_movepath_for_obstacle (int x, int y, int dir)
/* This function will check to see if an obstacle (like a wall) */
/* is in a tank's movement path.  If so it returns  TRUE.       */
{
 int new_x, new_y;     /* New coordinates for move         */

 /* set new coordinate's equal to original ones */
 new_x = x; new_y = y;

 switch (dir)
 {
   case (UP) :    new_y -= (TS+TANKMOVERATE);
		  break;
   case (RIGHT) : new_x += (TS+TANKMOVERATE);
		  break;
   case (DOWN)  : new_y += (TS+TANKMOVERATE);
		  break;
   case (LEFT)  : new_x -= (TS+TANKMOVERATE);
		  break;
 }



 /* Now check the new space for an obstacle */
  if (is_obstacle(new_x, new_y))
   {
    return TRUE;  /* obstacle is in tank's path */
   }

 /* Also check the space of where the treads are */
 if ((dir==UP) || (dir==DOWN))
   {
      if ((is_obstacle(new_x-TS,new_y)) ||
	  (is_obstacle(new_x+TS,new_y)))
	    return TRUE;
   }
 else if ((dir==LEFT) || (dir==RIGHT))
   {
	if ((is_obstacle(new_x,new_y+TS)) ||
	    (is_obstacle(new_x,new_y-TS)))
	      return TRUE;
   }

 /* Else if no obstacles were found, return FALSE */
 return FALSE;
}
/*************************************************************************/
int search_tankbox (int x, int y, int box_x, int box_y)
/* This function searches to see if (x,y) is in the box defined by   */
/* coordinates (box_x-TS, box_y-TS, box_x+TS, box_y+TS).             */
/*  It returns TRUE if it does.                                      */
{
 if ( ((x >= (box_x-(TS+2))) && (x <= (box_x+(TS+2)))) &&
      ((y >= (box_y-(TS+2))) && (y <= (box_y+(TS+2)))) )
	return TRUE;

 else
	return FALSE;
}
/*************************************************************************/
void scan_for_tank(int looktank,int lookdir,int *tankfound,int *enemynum,
		   int *distance)
/* This function takes in many parameters.                            */
/* Looktank holds the tanks array subscript number of the tank        */
/* conducting the searching.  Lookdir holds the direction which the   */
/* searching tank is currently looking.  Tankfound records True or    */
/* depending on whether or not an enemy tank is there. enemynum holds */
/* the subscript of an enemy tank if one is found.  And distance      */
/* simply holds the enumerated type of distance of the enemy tank.    */
{
 int i;
 int x,y;
 int x1,y1,x2,y2;

 *distance=0;
 *enemynum=-1;
 *tankfound=FALSE;

 for (i=0;((i<MAXTANKS) && (!*tankfound));i++)
  {
    /* Make sure its not looking for itself!, and target is alive */
    if ((i != looktank) && (tanks[i].tankstats[GAMESTATUS] == PLAYING))
     {
      /* Check to see if its an enemy tank  */
      if ((tanks[looktank].tankside) != (tanks[i].tankside))
       {
	 /* get both tanks (x,y) coordinates for checking */
	 x1=tanks[looktank].xcor;
	 y1=tanks[looktank].ycor;
	 x2=tanks[i].xcor;
	 y2=tanks[i].ycor;

	 /* Search in the correct direction */
	 switch(lookdir)
	  {
	   case UP :
	     for (y=y1;((y > (y1-SCANDISTANCE)) &&
		 (y > 0) && (!*tankfound)); y-=SCANRATE)
		  {
		    *tankfound=search_tankbox(x1,y,x2,y2);
		  }
	     break;

	   case RIGHT :
	     for (x=x1;((x < (x1+SCANDISTANCE)) &&
		 (x < GAME_GRID_WIDTH) && (!*tankfound)); x+=SCANRATE)
		  {
		    *tankfound=search_tankbox(x,y1,x2,y2);
		  }
	     break;

	   case DOWN  :
	     for (y=y1;((y < (y1+SCANDISTANCE)) &&
		 (y < GAME_GRID_HEIGHT) && (!*tankfound)); y+=SCANRATE)
		  {
		    *tankfound=search_tankbox(x1,y,x2,y2);
		  }
	     break;

	   case LEFT  :
	     for (x=x1;((x > (x1-SCANDISTANCE)) &&
		 (x > 0) && (!*tankfound)); x-=SCANRATE)
		  {
		    *tankfound=search_tankbox(x,y1,x2,y2);
		  }
	     break;

	  } /* end switch */
       } /* end if colors don't match */

     } /* end if i != looktank */

    if (*tankfound)
      *enemynum = i;

  } /* end for */


 if (*tankfound)    /* enemy tank is present, get the distance */
		   /* and return the array subscript of the   */
		   /* enemy tank.                             */
    {
     *distance = getdistance(tanks[looktank].xcor,tanks[looktank].ycor,
			     tanks[*enemynum].xcor,tanks[*enemynum].ycor);
     *distance = convertdistance(*distance);
    }
}
/*************************************************************************/
void locate_enemy(int *x, int *y,int attacker)
/* This function simply checks the tank array for an enemy tank  */
/* that is currently playing.  It then sends the x,y coordinates */
/* of this enemy back.                                           */
/* Note: this function always returns an enemy's coordinates,    */
/* this might need to be changed if the tank becomes invisible.  */
/* THINK ABOUT ADDING BELOW IDEA.  BUT WE DON'T WANT CONFUSED TANKS */
/* NOTE we must toggle the loop increments from + to - so that   */
/* the terminators don't just always hunt player one (0).        */
{
 int i;

  for (i=0;i<MAXTANKS;i++)
  {
    /* if the tanks are not on the same side */
    if ((tanks[attacker].tankside != tanks[i].tankside) &&
	(tanks[i].tankstats[GAMESTATUS]==PLAYING))
     {
       *x=tanks[i].xcor;   /* then this is an enemy tank, so */
       *y=tanks[i].ycor;   /* send the coordinates back.     */
       return;
     }
  }
}
/*************************************************************************/
int right_of(int dir)
/* This function simply returns what direction  */
/* is to the right of the passed in direction.  */
{
  if (dir==LEFT)
       return UP;
  else if (dir==UPLEFT)
       return UPRIGHT;
  else
       return (dir+2);
}
/*************************************************************************/
int left_of(int dir)
/* This function simply returns what direction  */
/* is to the left of the passed in direction.  */
{
  if (dir==UP)
       return LEFT;
  else if (dir==UPRIGHT)
       return UPLEFT;
  else
       return (dir-2);
}
/*************************************************************************/
int opposite_of(int dir)
/* This function simply returns what direction  */
/* is directly behind the passed in direction.  */
{
 if (dir < DOWNRIGHT)
     return (dir+4);
 else
     return (dir-4);
}
/*************************************************************************/
void turn_tank (int tanknum, int dir)
/* This function turns a tank to the desired direction (dir) by  */
/* calling a function which will store the needed moves into the */
/* tank's move buffer.                                           */
{
  switch (dir)
  {
    case UP : turn_tank_up(tanknum); break;
    case DOWN: turn_tank_down(tanknum); break;
    case LEFT: turn_tank_left(tanknum); break;
    case RIGHT: turn_tank_right(tanknum); break;
  }
}
/*************************************************************************/
void turn_tank_up (int tanknum)
/* This function will program the tank tanknum to point upwards */
{
 switch (tanks[tanknum].pos)
  {
    case UP: break;
    case UPRIGHT: add_buffer(tanknum,LEFTARROW); break;
    case RIGHT: add_buffer(tanknum,LEFTARROW);
		add_buffer(tanknum,LEFTARROW);  break;
    case DOWNRIGHT: add_buffer(tanknum,LEFTARROW);
		    add_buffer(tanknum,LEFTARROW);
		    add_buffer(tanknum,LEFTARROW);  break;
    case DOWN: add_buffer(tanknum,LEFTARROW);
	       add_buffer(tanknum,LEFTARROW);
	       add_buffer(tanknum,LEFTARROW);
	       add_buffer(tanknum,LEFTARROW); break;
    case DOWNLEFT: add_buffer(tanknum,RIGHTARROW);
		   add_buffer(tanknum,RIGHTARROW);
		   add_buffer(tanknum,RIGHTARROW);break;
    case LEFT: add_buffer(tanknum,RIGHTARROW);
	       add_buffer(tanknum,RIGHTARROW);
	       break;
    case UPLEFT: add_buffer(tanknum,RIGHTARROW);break;
  }
}
/*************************************************************************/
void turn_tank_down (int tanknum)
/* This function will program the tank (*tankis) to point downwards */
{
 switch (tanks[tanknum].pos)
  {
    case DOWNLEFT: add_buffer(tanknum,LEFTARROW); break;
    case LEFT: add_buffer(tanknum,LEFTARROW);
	       add_buffer(tanknum,LEFTARROW);break;
    case UPLEFT: add_buffer(tanknum,LEFTARROW);
		 add_buffer(tanknum,LEFTARROW);
		 add_buffer(tanknum,LEFTARROW);break;
    case UP: add_buffer(tanknum,LEFTARROW);
	     add_buffer(tanknum,LEFTARROW);
	     add_buffer(tanknum,LEFTARROW);
	     add_buffer(tanknum,LEFTARROW); break;
    case UPRIGHT: add_buffer(tanknum,RIGHTARROW);
		  add_buffer(tanknum,RIGHTARROW);
		  add_buffer(tanknum,RIGHTARROW);break;
    case RIGHT: add_buffer(tanknum,RIGHTARROW);
		add_buffer(tanknum,RIGHTARROW);break;
    case DOWNRIGHT: add_buffer(tanknum,RIGHTARROW);break;
    case DOWN:break;
  }
}
/*************************************************************************/
void turn_tank_left (int tanknum)
/* This function will program the tank (*tankis) to point leftwards */
{
 switch (tanks[tanknum].pos)
  {
    case UPLEFT: add_buffer(tanknum,LEFTARROW); break;
    case UP: add_buffer(tanknum,LEFTARROW);
	     add_buffer(tanknum,LEFTARROW); break;
    case UPRIGHT: add_buffer(tanknum,LEFTARROW);
		  add_buffer(tanknum,LEFTARROW);
		  add_buffer(tanknum,LEFTARROW); break;
    case RIGHT:   add_buffer(tanknum,LEFTARROW);
		  add_buffer(tanknum,LEFTARROW);
		  add_buffer(tanknum,LEFTARROW);
		  add_buffer(tanknum,LEFTARROW); break;
    case DOWNRIGHT: add_buffer(tanknum,RIGHTARROW);
		    add_buffer(tanknum,RIGHTARROW);
		    add_buffer(tanknum,RIGHTARROW);break;
    case DOWN:  add_buffer(tanknum,RIGHTARROW);
		add_buffer(tanknum,RIGHTARROW);break;
    case DOWNLEFT:  add_buffer(tanknum,RIGHTARROW); break;
    case LEFT: break;
  }
}
/*************************************************************************/
void turn_tank_right (int tanknum)
/* This function will program the tank (*tankis) to point rightwards */
{
 switch (tanks[tanknum].pos)
  {
    case UPRIGHT: add_buffer(tanknum,RIGHTARROW); break;
    case UP:    add_buffer(tanknum,RIGHTARROW);
		add_buffer(tanknum,RIGHTARROW); break;
    case UPLEFT: add_buffer(tanknum,RIGHTARROW);
		add_buffer(tanknum,RIGHTARROW);
		add_buffer(tanknum,RIGHTARROW); break;
    case LEFT: 	add_buffer(tanknum,RIGHTARROW);
		add_buffer(tanknum,RIGHTARROW);
		add_buffer(tanknum,RIGHTARROW);
		add_buffer(tanknum,RIGHTARROW); break;
    case DOWNLEFT: add_buffer(tanknum,LEFTARROW);
		   add_buffer(tanknum,LEFTARROW);
		   add_buffer(tanknum,LEFTARROW);
    case DOWN:	   add_buffer(tanknum,LEFTARROW);
		   add_buffer(tanknum,LEFTARROW); break;
    case DOWNRIGHT: add_buffer(tanknum,LEFTARROW); break;
    case RIGHT: break;
  }
}
/*************************************************************************/
void negotiate_obstacle (int tanknum)
/* This function will guide the inputed tank (*tankis) around an */
/* obstacle. It does this by checking the width of an obstacle   */
/* that is in front of its turret.  It will also note the depth. */
/* It will then determine if it should go around the obstacle to */
/* the left, or to the right.  All moves will be programmed into */
/* the tank's input buffer.                                      */
{
  int i;
  int x = tanks[tanknum].xcor;
  int y = tanks[tanknum].ycor;
  int leftdir, rightdir, forwarddir;
  int leftwidth,rightwidth,depth;
  int obstacle_left = FALSE;
  int obstacle_right = FALSE;

  /* Calculate which directions are to the left and right */
  leftdir = left_of (tanks[tanknum].pos);
  rightdir = right_of (tanks[tanknum].pos);
  forwarddir = tanks[tanknum].pos;

  /* Now calculate the width & depth (in TANKMOVERATE denominations) */
  leftwidth= get_width_obstacle (x,y,tanks[tanknum].pos,leftdir);
  rightwidth= get_width_obstacle (x,y,tanks[tanknum].pos,rightdir);
  depth= get_width_obstacle (x,y,tanks[tanknum].pos,forwarddir);

  /* Now see if any obstacles block a way */
  obstacle_left = check_movepath_for_obstacle (x,y,leftdir);
  obstacle_right = check_movepath_for_obstacle (x,y,rightdir);

  /* Now determine which way to go */
 if (((leftwidth <= rightwidth) && (!obstacle_left)) ||
     ((rightwidth < leftwidth) && (obstacle_right)))
  {
   add_buffer(tanknum, LEFTARROW); /* Make a left turn */
   add_buffer(tanknum, LEFTARROW);
   for (i=0; i < leftwidth; i++)
     add_buffer(tanknum, UPARROW);
   add_buffer(tanknum, RIGHTARROW); /* Turn back to original direction */
   add_buffer(tanknum, RIGHTARROW);
   for (i=0; i < depth; i++)
     add_buffer(tanknum, UPARROW);
  }

 else if (((rightwidth < leftwidth) && (!obstacle_right)) ||
	 ((leftwidth <= rightwidth) && (obstacle_left)))
  {
   add_buffer(tanknum, RIGHTARROW);  /* Make a right turn */
   add_buffer(tanknum, RIGHTARROW);
   for (i=0; i < rightwidth; i++)
     add_buffer(tanknum, UPARROW);
   add_buffer(tanknum, LEFTARROW);  /* Turn back to original direction */
   add_buffer(tanknum, LEFTARROW);
   for (i=0; i < depth; i++)
     add_buffer(tanknum, UPARROW);
  }
 /* Maybe add an else if here, so if an obstacle exists where */
 /* the tank can get trapped in, except from behind           */
}
/*************************************************************************/
int get_width_obstacle (int x, int y, int pos, int dir)
/* This function returns the width of an obstacle according to the */
/* (x,y) position passed in of the tank. The width returned is in  */
/* TANKMOVERATE denominations.                                     */
{
 int obstacle_x, obstacle_y;
 int x_step=0;
 int y_step=0;
 int width=0;
 int tankmoverate_width=0;

 /* Set the obstacle's x,y coordinates equal to the tanks */
 obstacle_x=x; obstacle_y=y;

 /* Evaluate the obstacles closest x,y coordinate to the tank */
 switch (pos)  /* the direction the tank is facing */
 {
   case (UP) :    obstacle_y -= (TS+TANKMOVERATE);
		  break;
   case (RIGHT) : obstacle_x += (TS+TANKMOVERATE);
		  break;
   case (DOWN)  : obstacle_y += (TS+TANKMOVERATE);
		  break;
   case (LEFT)  : obstacle_x -= (TS+TANKMOVERATE);
		  break;
 }

 /* Now get the width of the obstacle according to the direction */
 /* (dir) passed in as a parameter.                              */

 /* First calculate the step rates (i.e. the width direction to check) */
   switch (dir)
   {
     case (UP) : y_step = -1; break;
     case (DOWN) : y_step = +1; break;
     case (LEFT) : x_step = -1; break;
     case (RIGHT) : x_step = +1; break;
   }

 /* Now, get the width */
 while ( is_obstacle(obstacle_x,obstacle_y) &&
	 (obstacle_x > 0) && (obstacle_x < GAME_GRID_WIDTH) &&
	(obstacle_y > 0) && (obstacle_y < GAME_GRID_HEIGHT))
 {
   obstacle_x += x_step;  /* move along the obstacle's x coordinate */
   obstacle_y += y_step;  /* move along the obstacle's y coordinate */

   width += (x_step + y_step);   /* accumulate the width */
 }

 /* Make sure you don't return a negative width */
  if (width < 0)
    width *= -1;

 /* Put into TANKMOVERATE denominations, without using division */
  while (width >= 0)
  {
    width = width - TANKMOVERATE;
    tankmoverate_width++;
  }

 return (tankmoverate_width+2);  /* +2 for to make sure tank clears */
}                                /* the edge of the obstacle.       */
/*************************************************************************/
int examine_terrain(int x, int y)
/* This function takes in the x,y coordinate of a tank, and calculates */
/* eight points around the tank to see if obstacles are present. It    */
/* then returns the estimated type of terrain the tank is in           */
{
 int obstacle_count=0;
						   /* Check Eight Points */
 obstacle_count += is_obstacle (x+(TSMUL2),y);      /*   x    x     x   */
 obstacle_count += is_obstacle (x-(TSMUL2),y);      /*                  */
 obstacle_count += is_obstacle (x,y+(TSMUL2));      /*   x  (tank)  x   */
 obstacle_count += is_obstacle (x,y-(TSMUL2));      /*                  */
						    /*   x    x     x   */

 obstacle_count += is_obstacle (x+(TSMUL2),y+(TSMUL2));
 obstacle_count += is_obstacle (x-(TSMUL2),y-(TSMUL2));
 obstacle_count += is_obstacle (x-(TSMUL2),y+(TSMUL2));
 obstacle_count += is_obstacle (x+(TSMUL2),y-(TSMUL2));

 if (obstacle_count==0)
   return EMPTY;
 else if ((obstacle_count > 0) && (obstacle_count < 4))
   return SPOTTED;
 else
   return DENSE;
}
/*************************************************************************/
int closest_obstacle_is(int tank_x, int tank_y)
/* This function returns an integer which represents the direction */
/* of the closest obstacle from the tank at (tank_x, tank_y).      */
{
  int x1,x2,y1,y2;
  int ob_found=FALSE;   /* obstacle found */

  x1=x2=tank_x;
  y1=y2=tank_y;

   do
   {
     x1+=SCANRATE; x2-=SCANRATE; y1+=SCANRATE; y2-=SCANRATE;

     /* look up */
     if (is_obstacle(tank_x,y2))  return UP;
     /* look down */
     else if (is_obstacle(tank_x,y1)) return DOWN;
     /* look right */
     else if (is_obstacle(x1,tank_y)) return RIGHT;
     /* look left */
     else if (is_obstacle(x2,tank_y)) return LEFT;
   }
   while ((!ob_found) && (x1<GAME_GRID_WIDTH) && (x1>0) && (y1>0) &&
	 (y1<GAME_GRID_HEIGHT) && (x2<GAME_GRID_WIDTH) && (x2>0) && (y2>0)
	 && (y2<GAME_GRID_HEIGHT));


 return 0;  /* nothing found */
}
/*************************************************************************/
void randomattack (int tanknum)
/* This function simply rolls a die and determines what move the   */
/* computer tank will do.  It is completely random so the computer */
/* tank will not pose as much of an opponent.                      */
/* This function is used for a tank set to RANDOM attack mode.     */
{
 int dieroll;

 /* roll die for number between 1-16 */
 dieroll = (rand() % 16 + 1);


 if (dieroll==1)   /* do nothing */
  {
   tanks[tanknum].currentinput = 0;
  }
 else if ((dieroll > 1) && (dieroll < 10))  /* move forward */
  {
   tanks[tanknum].currentinput = UPARROW;
  }
 else if (dieroll==10)    /* move backward */
  {
   tanks[tanknum].currentinput = DOWNARROW;
  }
 else if ((dieroll > 10) && (dieroll < 13))  /* rotate left */
  {
   tanks[tanknum].currentinput = LEFTARROW;
  }
 else if ((dieroll > 12) && (dieroll < 15))   /* rotate right */
  {
   tanks[tanknum].currentinput = RIGHTARROW;
  }
 else /* 15 through 16 */ /* fire a projectile */
  {
   tanks[tanknum].currentinput = SPACE;
  }
}
/***************************************************************************/
#define  _TANKSRCH
#endif

